/*
  mtetherd
  (c) 2010 Gregor Riepl <onitake@gmail.com>
  
  Tethering utility for Maemo
  
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _MTETHERD_NET_H
#define _MTETHERD_NET_H

#include <glib.h>
#include <glib-object.h>

G_BEGIN_DECLS

#define TYPE_MTETHERD_DEVICE (mtetherd_device_get_type())
#define MTETHERD_DEVICE(object) (G_TYPE_CHECK_INSTANCE_CAST((object), TYPE_MTETHERD_DEVICE, MTetherDDevice))
#define MTETHERD_DEVICE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass), TYPE_MTETHERD_DEVICE, MTetherDDeviceClass))
#define IS_MTETHERD_DEVICE(object) (G_TYPE_CHECK_INSTANCE_TYPE((object), TYPE_MTETHERD_DEVICE))
#define IS_MTETHERD_DEVICE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), TYPE_MTETHERD_DEVICE))
#define MTETHERD_DEVICE_GET_CLASS(object) (G_TYPE_INSTANCE_GET_CLASS((object), TYPE_MTETHERD_DEVICE, MTetherDDeviceClass))

typedef struct _MTetherDDeviceClass MTetherDDeviceClass;
typedef struct _MTetherDDevice MTetherDDevice;
typedef struct _MTetherDDevicePrivate MTetherDDevicePrivate;

struct _MTetherDDeviceClass {
	GObjectClass parent;
};

struct _MTetherDDevice {
	GObject parent;
	MTetherDDevicePrivate *priv;
};

GType mtetherd_device_get_type();

// Allocates a new device object
// This object doesn't have addresses assigned yet
// Add it to a device list or use mtetherd_device_set_index to construct them
MTetherDDevice *mtetherd_device_new(const gchar *interface, const gchar *udi);
// Constructs IP address and DHCP ranges from the given index
// 192.168.255.0/24 is the supernet for all devices,
// each gets a /30 from that range, depending on the index
// Example: index = 32 -> subnet = 192.168.255.200/30
// The host address is always 1, and the DHCP range goes from 1 to 2
void mtetherd_device_set_index(MTetherDDevice *self, guint index);
// Returns the interface name
const gchar *mtetherd_device_get_interface(MTetherDDevice *self);
// Returns the host address
// Free when done
gchar *mtetherd_device_get_addr(MTetherDDevice *self);
// Returns the netmask
// Free when done
gchar *mtetherd_device_get_netmask(MTetherDDevice *self);
// Returns the DHCP range start address
// Free when done
gchar *mtetherd_device_get_dhcp_start(MTetherDDevice *self);
// Returns the DHCP range end address
// Free when done
gchar *mtetherd_device_get_dhcp_end(MTetherDDevice *self);

// Device list functions
// The list is constrained to 64 entries
gpointer mtetherd_device_list_new();
// Unrefs all devices contained within and frees the list
void mtetherd_device_list_free(gpointer list);
// Scans the list for an udi and returns the first device found
MTetherDDevice *mtetherd_device_list_find(gpointer list, const gchar *udi);
// Adds a device to the next free position
// This transfers ownership of the object to the list
// Returns FALSE if the list is already full
gboolean mtetherd_device_list_add(gpointer list, MTetherDDevice *device);
// Removes and deallocates the first device with the given UDI from the list
// Returns FALSE if the UDI is not found
gboolean mtetherd_device_list_remove(gpointer list, const gchar *udi);

// Returns true if the device is acceptable for tethering
// Currently checks if the interface name starts with usb or bnep
gboolean mtetherd_device_ok(const gchar *interface);

G_END_DECLS

#endif //_MTETHERD_NET_H

